@@ -28,6 +28,8 @@ def split(sequence: List[T], parts: int) -> List[List[T]]:
 def get_equal_bins(probs: List[float], num_bins: int=10) -> Bins:
     """Get bins that contain approximately an equal number of data points."""
     sorted_probs = sorted(probs)
+    if num_bins > len(sorted_probs):
+        num_bins = len(sorted_probs)
     binned_data = split(sorted_probs, num_bins)
     bins: Bins = []
     for i in range(len(binned_data) - 1):
@@ -192,10 +194,30 @@ def get_binning_ce(probs, labels, p=2, debias=True, mode='marginal'):
 
 
 def get_ece(probs, labels, debias=False, num_bins=15, mode='top-label'):
+    """Get ECE as computed by Guo et al."""
     return lower_bound_scaling_ce(probs, labels, p=1, debias=debias, num_bins=num_bins,
                                   binning_scheme=get_equal_prob_bins, mode=mode)
 
 
+def get_ece_em(probs, labels, debias=False, num_bins=15, mode='top-label'):
+    """Get ECE, but use equal mass binning."""
+    return lower_bound_scaling_ce(probs, labels, p=1, debias=debias, num_bins=num_bins,
+                                  binning_scheme=get_equal_bins, mode=mode)
+
+
+def get_selective_stats(probs, correct):
+    """Return area under coverage-accuracy curve, and acc for 10% most confident predictions."""
+    # Sort in descending order.
+    probs = np.array(probs)
+    correct = np.array(correct)
+    sort_indices = np.argsort(-probs)
+    sorted_correct = correct[sort_indices]
+    accs = np.cumsum(sorted_correct) / np.arange(1, len(sorted_correct) + 1)
+    coverage_acc_area = np.mean(accs)
+    acc_percentile_90 = accs[int(0.1 * len(sorted_correct))]
+    return coverage_acc_area, acc_percentile_90
+
+
 def _get_ce(probs, labels, p, debias, num_bins, binning_scheme, mode='marginal'):
     def ce_1d(probs, labels):
         assert probs.shape == labels.shape
@@ -224,11 +246,11 @@ def _get_ce(probs, labels, p, debias, num_bins, binning_scheme, mode='marginal')
         raise ValueError('labels and probs should have the same number of entries.')
     if len(probs.shape) == 1:
         # If 1D (2-class setting), compute the regular calibration error.
-        if np.min(labels) != 0 or np.max(labels) != 1:
+        if np.min(labels) < 0 or np.max(labels) > 1:
             raise ValueError('If probs is 1D, each label should be 0 or 1.')
         return ce_1d(probs, labels)
     elif len(probs.shape) == 2:
-        if np.min(labels) != 0 or np.max(labels) != probs.shape[1] - 1:
+        if np.min(labels) < 0 or np.max(labels) > probs.shape[1] - 1:
             raise ValueError('labels should be between 0 and num_classes - 1.')
         if mode == 'marginal':
             labels_one_hot = get_labels_one_hot(labels, k=probs.shape[1])
@@ -504,8 +526,8 @@ def get_accuracy(probs, labels):
 
 
 def get_labels_one_hot(labels, k):
-    assert np.min(labels) == 0
-    assert np.max(labels) == k - 1
+    assert np.min(labels) >= 0
+    assert np.max(labels) <= k - 1
     num_labels = labels.shape[0]
     labels_one_hot = np.zeros((num_labels, k))
     labels_one_hot[np.arange(num_labels), labels] = 1
